<?php /* datastore.php
This file is part of BlogTorrent http://www.blogtorrent.com/
nassar 'at' downhillbattle 'dot' org
Licensed under the terms of the GNU GPL */

require_once("zipfile.php");

/* Stores data in a flat file database
   Tracker info is stored in binary format using 7 bytes per client
  in the following format:
  [Seeder: 1 bit][Time: 7bits][IP: 4 bytes][Port: 2 bytes]

  User info is stored in a bencoded file
*/
class FlatFileStore {
  var $error;

  function type() {
    return 'flat file';
  }

  //Takes in parameters for a BitTorrent announce
  //returns data to be passed back to the client or NULL on error
  function BTAnnounce($info_hash,$event,$remote_addr,$port,$left,$numwant) {

    $this->error = '';
    if(strlen($info_hash) != 40)
      {
	$this->error = 'Invalid info hash';
	return null;
      }
    $peer_ip = explode('.', $remote_addr);
    $peer_ip = pack("C*", $peer_ip[0], $peer_ip[1], $peer_ip[2], $peer_ip[3]);
    $peer_port = pack("n*", (int)$port);
    //Generate a number 0-127 based on the minute
    $time = intval((time() % 7680) / 60);
    //If this is a seeder, set the high bit
    if($left == 0)
      {
	$time += 128;
      }
    $time = pack("C", $time);

    if(!file_exists('data/'.$info_hash))
      {
	$this->error = 'This torrent is not authorized on this tracker.';
	return null;
      }
    $handle = fopen('data/'.$info_hash, "rb+");
    flock($handle, LOCK_EX);
    $peer_num = intval(filesize('data/'.$info_hash) / 7);
    if ($peer_num>0)
      $data = fread($handle, $peer_num * 7);
    else
      $data = '';
    $peer = array();
    $updated = false;

    //Update the peer
    for($i=0;$i<$peer_num;$i++)
      {
	if(($peer_ip . $peer_port) == substr($data, $i * 7 + 1, 6))
	  {
	    $updated = true;
	    if($event == 'stopped')
	      {
		$peer_num--;
	      }
	    else
	      {
		$peer[$i] = $time . $peer_ip . $peer_port;
	      }
	  }
	else
	  {
	    $peer[$i] = substr($data, $i * 7, 7);
	  }
      }
    //If we don't already have this peer in that database, add it
    if($updated == false)
      {
	$peer[] = $time . $peer_ip . $peer_port;
	$peer_num++;
      }
    rewind($handle);
    ftruncate($handle, 0);
    fwrite($handle, join('', $peer), $peer_num * 7);
    flock($handle, LOCK_UN);
    fclose($handle);

    $o = '';
    //Fill $o with a list of peers
    if ($event == 'stopped' || $numwant === 0) {
      $o = '';
    } else {
      if($peer_num > 50)
	{
	  $key = array_rand ($peer, 50);
	  foreach($key as $val)
	    {
		$o .= substr($peer[$val], 1, 6);
	    }
	}
      else
	{
	  for($i=0;$i<$peer_num;$i++)
	    {
	      $o .= substr($peer[$i], 1, 6);
	    }
	}
    }
    return 'd8:intervali1800e5:peers' . strlen($o) . ':' . $o . 'e';
  }

  function getStat($info_hash) {
    $complete = 0;
    $incomplete = 0;
    global $time;
    $handle = fopen('data/'.$info_hash, "rb");
    flock($handle, LOCK_EX);
    $size = filesize('data/'.$info_hash);
    if ($size>0) {
      $x = fread($handle, $size);
      flock($handle, LOCK_UN);
      fclose($handle);
      $no_peers = intval(strlen($x) / 7);
      for($j=0;$j<$no_peers;$j++)
	{
	  $t_peer_seed = join('', unpack("C", substr($x, $j * 7, 1)));
	  if($t_peer_seed >= 128)
	    {
	      $complete++;
	    }
	  else
	    {
	      $incomplete++;
	    }
	}
    }
    return array("hash" => $info_hash,
		 "complete" => $complete,
		 "incomplete" => $incomplete);
  }


function getHashFromTorrent($filename) {
  global $store;
  $tmp = $store->getTorrent($filename);

  return $tmp["sha1"];
}

  function getTorrentList() {
    $list = array();
    $times = array();
    $handle = opendir('./torrents/');
    while (false !== ($torrentfile = readdir($handle)))
      {
	if (($torrentfile != '.') && ($torrentfile != '..') &&
	    ($torrentfile != '.htaccess')) {
	  //$torrent = bdecode(file_get_contents('./torrents/'.$torrentfile));
	  $list[] = $torrentfile;
	  $times[] = $this->getTorrentDate($torrentfile);
	}
      }
    if (count($list) > 0) {
	    array_multisort($times, SORT_DESC, $list);
	}
    return $list;
  }

  function getTorrent($filename) {
    return bdecode($this->getRawTorrent($filename));
  }

  function getRawTorrent($filename) {
    return file_get_contents('./torrents/'.$filename);
  }

  function getTorrentDate($filename) {
  	return filectime('./torrents/' . $filename);
  }

  function torrentExists($info_hash) {
    return file_exists('data/'.$info_hash);
  }

  function getTorrentDetails($info_hash) {
    $peers = array();

    $handle = fopen('data/'.$info_hash, "rb+");
    flock($handle, LOCK_EX);
    if (filesize('data/'.$info_hash)>0)
      $x = fread($handle, filesize('data/'.$info_hash));
    else
      $x = '';
    flock($handle, LOCK_UN);
    fclose($handle);
    $no_peers = intval(strlen($x) / 7);
    for($j=0;$j<$no_peers;$j++)
      {
	$ip = unpack("C*", substr($x, $j * 7 + 1, 4));
	$ip = $ip[1] . '.' . $ip[2] . '.' . $ip[3] . '.*';
	$port = join('', unpack("n*", substr($x, $j * 7 + 5, 2)));
	$t_peer_seed = join('', unpack("C", substr($x, $j * 7, 1)));
	if($t_peer_seed >= 128)
	  {
	    $what = 'seeder';
	    $t_time = $t_peer_seed - 128;
	  }
	else
	  {
	    $what = 'leecher';
	    $t_time = $t_peer_seed;

	  }
	$time = intval((time() % 7680) / 60);
	$peers[] = array("ip" => $ip,
		     "what" => $what,
		     "port" => $port,
		     "time" => number_format($time - $t_time));

      }
    return $peers;
  }

  function isValidAuthHash($username,$hash) {
    $hashes=bdecode(file_get_contents('data/hash'));
    return (isset($hashes[sha1($username.$hash)]) &&
	  ($hashes[sha1($username.$hash)]>(time()-3600)));
  }

  function getAuthHash($username,$passhash) {
    $contents = '';

    $handle = fopen('data/hash', "ab+");
    fseek($handle,0);
    flock($handle,LOCK_EX);
    while (!feof($handle)) {
      $contents .= fread($handle, 8192);
    }
    $hashes=bdecode($contents);
    if (!is_array($hashes))
      $hashes = array();
    $hash = md5($username.microtime().rand().$passhash);
    $hashes[sha1($username.$hash)]=time();
    clear_old_authhashes($hashes);
    ftruncate($handle,0);
    fseek($handle,0);
    fwrite($handle,bencode($hashes));

    fclose($handle);
    return $hash;
  }

  function dropAuthHash($username,$hash) {
    $contents = '';
    $handle = fopen('data/hash', "a+b");
    fseek($handle,0);
    flock($handle,LOCK_EX);
    while (!feof($handle)) {
      $contents .= fread($handle, 8192);
    }
    $hashes=bdecode($contents);
    unset($hashes[sha1($username.$hash)]);
    $this->clearOldAuthHashes($hashes);
    ftruncate($handle,0);
    fseek($handle,0);
    fwrite($handle,bencode($hashes));
    fclose($handle);
  }

  function clearOldAuthHashes(&$hashes) {
    $now = time();
    foreach ($hashes as $hash => $stamp) {
      if ($stamp<$now-3600) //Hash was created more than 1 hour ago
	unset($hashes[$hash]);
    }
  }


  function settingsExist() {
    return file_exists('data/settings');
  }

//Sends the location of this feed to BlogTorrent.com periodically
  function phoneHome() {
    global $settings;
  //Phone home if settings have been saved, the box is checked, and we
  //haven't phoned home in a week
    if ($this->settingsExist() && $settings['Ping']) {
      if (file_exists('data/phonelock')) {
	$stat = stat('data/phonelock');
	$doit = $stat[9]<(time()-604800); //9 is mtime
      } else
	$doit = true;
      if ($doit) {
	$handle = fopen('data/phonelock','wb');
	fclose($handle);
	file_get_contents('http://www.blogtorrent.com/register.php?server='.htmlspecialchars(get_base_url()).'rss.php&version='.get_version());
      }
      return $doit;
    }
  }

  function addTorrentToTracker($tmpfile,$torrent) {
    global $seeder;
    global $settings;
    if (!file_exists('torrents/'.$torrent)) {
      move_uploaded_file($tmpfile,'torrents/'.$torrent);
      $info_hash = $this->getHashFromTorrent($torrent);
      if(!file_exists('data/'.$info_hash))
	{
	  $handle = fopen('data/'.$info_hash,"wb");
	  fclose($handle);
	}
      }
    if ($seeder->enabled() && $settings["sharing_auto"]) {
      $seeder->spawn($torrent);
    }

  }

  function getAllUsers() {
    if (!file_exists('data/users')||
	(filesize('data/users')==0))
      return array();
    else
      return bdecode(file_get_contents('data/users'));
  }

  function deleteTorrent($torrent) {
    global $seeder;
    if ($seeder->enabled())
      $seeder->stop($torrent);
    $file = $this->getHashFromTorrent($torrent);
    unlink("torrents/$torrent");
    unlink("data/$file");
  }

  function deleteUser($username) {
    $contents = '';
    $firstuser= (!file_exists('data/users'));

    $handle2 = fopen('data/users', "a+b");
    fseek($handle2,0);
    flock($handle2,LOCK_EX);
    if (!$firstuser) {
      $contents = '';
      while (!feof($handle2)) {
	$contents .= fread($handle2, 8192);
      }
      $users=bdecode($contents);
    } else {
      $users = array();
    }

    unset($users[$username]);

    fseek($handle2,0);
    ftruncate($handle2,0);
    fwrite($handle2,bencode($users));
    fclose($handle2);
  }

  //New users are stored as a list of hashes
  function addNewUser($username,$password,$email, &$error) {
    $contents = '';
    $users = $this->getAllUsers();
    if (isset($users[$username])) {
      $error = "That username already exists";
      return false;
    }
    foreach ($users as $user) {
      if ($email==$user['Email']) {
	$error = "A user with that email address is already registered";
	return false;
      }
    }

    $handle = fopen('data/newusers', "a+b");
    fseek($handle,0);
    flock($handle,LOCK_EX);
    while (!feof($handle)) {
      $contents .= fread($handle, 8192);
    }
    $newusers=bdecode($contents);
    $hashlink=sha1($username.$password.$email);
    $filehash=sha1($username.$hashlink);
    $newusers[$filehash]['Hash']=hashpass($username,$password);
    $newusers[$filehash]['Email']=$email;
    $newusers[$filehash]['Created']=time();

    ftruncate($handle,0);
    fseek($handle,0);
    fwrite($handle,bencode($newusers));
    fclose($handle);

    mail($email,"New Account on ".site_title(),"Click below to activate your account:\n".get_base_url()."login.php?hash=".$hashlink."&username=".$username);

    return true;
}

function authNewUser($hashlink,$username) {
  global $settings;
  $contents = '';
  $success = false;

  $filehash=sha1($username.$hashlink);

  $handle = fopen('data/newusers', "a+b");
  fseek($handle,0);
  flock($handle,LOCK_EX);
  while (!feof($handle)) {
    $contents .= fread($handle, 8192);
  }
  $newusers=bdecode($contents);

  if (isset($newusers[$filehash])) {
    $users = get_all_users();
    if (isset($users[$username])) {
      return false;
    }
    foreach ($users as $user) {
      if ($newusers[$filehash]['Email']==$user['Email']) {
	return false;
      }
    }

    $firstuser= (!file_exists('data/users'));

    $handle2 = fopen('data/users', "a+b");
    fseek($handle2,0);
    flock($handle2,LOCK_EX);
    if (!$firstuser) {
      $contents = '';
      while (!feof($handle2)) {
	$contents .= fread($handle2, 8192);
      }
      $users=bdecode($contents);
    } else {
      $users = array();
    }

    $users[$username]['Hash']=$newusers[$filehash]['Hash'];
    $users[$username]['Email']=$newusers[$filehash]['Email'];
    $users[$username]['CanUpload']=$settings['AutoAuthCanUpload'];
    $users[$username]['CanAdmin']=$firstuser;

    fseek($handle2,0);
    ftruncate($handle2,0);
    fwrite($handle2,bencode($users));
    fclose($handle2);

    $users[$username]['Username'] = $username;
    $_SESSION['user']=$users[$username];

    unset($newusers[$filehash]);
    $success = true;
  }

  ftruncate($handle,0);
  fseek($handle,0);
  fwrite($handle,bencode($newusers));
  fclose($handle);

  return $success;
}

function changePassword($password) {
  $contents = '';
  if (is_array($_SESSION['user'])) {
    $username= $_SESSION['user']['Username'];

    $handle = fopen('data/users', "a+b");
    fseek($handle,0);
    flock($handle,LOCK_EX);
    while (!feof($handle)) {
      $contents .= fread($handle, 8192);
    }
    $users=bdecode($contents);

    if (isset($users[$username])) {
      $users[$username]['Hash']=hashpass($username,$password);
    }

    fseek($handle,0);
    ftruncate($handle,0);
    fwrite($handle,bencode($users));
    fclose($handle);
    return true;
  } else
    return false;
}

//Adds/Updates a user in the user database
 function updateUser($username,$hash,$email,$canUpload=false,$canAdmin=false) {
   $contents = '';
   if (!is_admin())
     return false;

   $handle = fopen('data/users', "a+b");
   fseek($handle,0);
   flock($handle,LOCK_EX);
   while (!feof($handle)) {
     $contents .= fread($handle, 8192);
   }
   $users=bdecode($contents);
   $users[$username]['Hash']=$hash;
   $users[$username]['Email']=$email;
   $users[$username]['CanUpload']=$canUpload;
   $users[$username]['CanAdmin']=$canAdmin;

   fseek($handle,0);
   ftruncate($handle,0);
   fwrite($handle,bencode($users));
   fclose($handle);
 }

  function setup() {
    error_reporting(0);
    if ((!file_exists('data') && !mkdir('data',0700)) ||
	(!file_exists('torrents') && !mkdir('torrents',0700)) ||
	(!is_writable('data')) ||
	(!is_writable('torrents'))){
      return false;
    }
    if (!file_exists('data/.htaccess')) {
      $file = fopen('data/.htaccess','wb');
      fwrite($file,"deny from all\n");
      fclose($file);
    }
    if (!file_exists('torrents/.htaccess')) {
      $file = fopen('torrents/.htaccess','wb');
      fwrite($file,"deny from all\n");
      fclose($file);
    }
    if ((!file_exists('macclient.obj')) &&
	(!file_exists('data/macclient.obj'))) {
      $data = $this->createZipObject(preg_replace('|^(.*[\\/]).*$|','\\1',$_SERVER['SCRIPT_FILENAME']).'BlogTorrentMac.zip');
      if (is_null($data))
	die ("Mac executable not found. Please, copy macclient.obj to your web directory");
      $file = fopen('data/macclient.obj','wb');
      fwrite($file,$data);
      fclose($file);
    }
    //Production servers should probably comment this out
    error_reporting(E_ALL);
    return $this->loadSettings();
  }

  //Converts a zipfile into a serialized PHP object

  //Normally, we ship Blog Torrent with a serialized object containing
  //the Mac client, but if it's not there, we can make it on the fly
  //using this function
  function createZipObject($zipfile) {
    $zipobj = new zipfile();
    $origzip = zip_open($zipfile);

    if ($origzip) {
      while ($entry = zip_read($origzip)) {
	$name = zip_entry_name($entry);
	$size = zip_entry_filesize($entry);
	if ($size == 0)
	  $zipobj->add_dir($name);
	else {
	  zip_entry_open($origzip,$entry);
	  $data = zip_entry_read($entry,$size);
	  zip_entry_close($entry);
	  $zipobj->add_file($data,$name,9);
	}
      }
      return serialize($zipobj);
    } else {
      return null;
    }
  }

//Loads settings from config file
//Returns true on success, false on failure
  function loadSettings() {
    global $settings;
    $contents = '';

    if (!file_exists('data/settings') ||
	(filesize('data/settings')==0))
      $settings = array('Title'=>$_SERVER['HTTP_HOST'].' Torrents',
			'Description'=>$_SERVER['HTTP_HOST'].' Blog Torrent tracker',
			'NonAuthUpload'=>true,
			'AutoAuthCanUpload'=>true,
			'Ping'=>true,
			'sharing_enable' => false,
			'sharing_auto' => true,
			'sharing_python' => '',
			'sharing_actual_python' => '',
			'HideNonSeed' => false);
    else {
      $handle = fopen('data/settings', "rb");
      flock($handle,LOCK_EX);
      while (!feof($handle)) {
	$contents .= fread($handle, 8192);
      }
      $settings=bdecode($contents);

      //Early beta's didn't have these settings
      if (!isset($settings['Ping']))
	$settings['Ping']=true;
      if (!isset($settings['sharing_enable']))
	$settings['sharing_enable'] = false;
      if (!isset($settings['sharing_auto']))
	$settings['sharing_auto'] = true;
      if (!isset($settings['sharing_python']))
	$settings['sharing_python'] = '';
      if (!isset($settings['sharing_actual_python']))
	$settings['sharing_actual_python'] = '';
      if (!isset($settings['HideNonSeed']))
        $settings['HideNonSeed'] = false;
      fclose($handle);
    }
    return true;

  }

  //Saves settings to config file
  //Returns true on success, false on failure
  function saveSettings($newsettings) {
    global $settings;
    if (!is_admin())
      return false;

    $handle = fopen('data/settings', "a+b");
    flock($handle,LOCK_EX);
    fseek($handle,0);
    ftruncate($handle,0);
    $settings=$newsettings;
    fwrite($handle,bencode($settings));
    fclose($handle);
    return true;
  }

   function setupHelpMessage() {
    $output = <<<EOD
<html>
    <head>
        <title>Blog Torrent Installation</title>
        <link rel="stylesheet" type="text/css" href="basic.css" />
        <link rel="alternate" type="application/rss+xml" title="RSS" href="rss.php" />
    </head>
<body>
<div id="rap">
<h1>Blog Torrent Installation</h1>
<p>Blog Torrent is almost installed. Unfortunately, I can't do the last step of the installation automatically, so you'll have to do it by hand. You need to create the data directories for Blog Torrent. To do this:</p>

<ol>
    <li>Make directories named "torrents/" and "data/"</li>
    <li>Change the permissions on those directories to be writable by the web server</li>
</ol>

<p>On a typical web server, you might do this by connecting with your ftp client and typing the following commands:</p>

<pre>
EOD;
   $output .= "cd ".preg_replace('|^(.*[\\/]).*$|','\\1',$_SERVER['SCRIPT_FILENAME']);

   $output .= <<<EOD

mkdir data
mkdir torrents
chmod 777 data
chmod 777 torrents
</pre>

<p>Note that giving the directories "777" permissions will allow
anyone on the server to full access those directories. If you share a
server with others, they may be able to tamper with you Blog Torrent
data files if you use these settings. There may be other settings more
appropriate for your server. <b>Please, contact your system administrator
if you have any questions about permissions.</b></p>

<p>Once you've installed Blog Torrent, you should create a user account by
going to newuser.php, then configure Blog Torrent using settings.php.</p>
</div>
</body>
</html>
EOD;

  return $output;
   }

}

class MYSqlStore extends FlatFileStore {
   function BTAnnounce($info_hash,$event,$remote_addr,$port,$left,$numwant) {
     $this->error = '';
     if(strlen($info_hash) != 40)
       {
	 $this->error = 'Invalid info hash';
	 return null;
       }
     $peer_ip = explode('.', $remote_addr);
     $peer_ip = pack("C*", $peer_ip[0], $peer_ip[1], $peer_ip[2], $peer_ip[3]);
     $peer_port = pack("n*", (int)$port);
     $seeder = ($left == 0)?'1':'0';

     if(!$this->torrentExists($info_hash))
      {
	$this->error = 'This torrent is not authorized on this tracker.';
	return null;
      }
     if($event == 'stopped')
       mysql_query("DELETE FROM peers WHERE info_hash='".mysql_escape_string($info_hash)."' AND ip='".mysql_escape_string($peer_ip)."' AND port='".mysql_escape_string($peer_port)."'");
     else
       mysql_query("REPLACE INTO peers (info_hash,ip,port,seeder,time) VALUES ('".mysql_escape_string($info_hash)."', '".mysql_escape_string($peer_ip)."','".mysql_escape_string($peer_port)."','".mysql_escape_string($seeder)."',NOW())");

    $o = '';
    //Fill $o with a list of peers
    if ($event == 'stopped' || $numwant === 0) {
      $o = '';
    } else {
      $result = mysql_query("SELECT CONCAT(ip,port) as out FROM peers WHERE info_hash='".mysql_escape_string($info_hash)."' ORDER BY RAND() LIMIT 50");
      while ($row = mysql_fetch_array($result))
	$o.= $row[0];
    }
    return 'd8:intervali1800e5:peers' . strlen($o) . ':' . $o . 'e';

   }
  function getStat($info_hash) {
    $row = mysql_fetch_array(mysql_query("SELECT COUNT(*) FROM peers WHERE info_hash='".mysql_escape_string($info_hash)."'"));
    $total = $row[0];

    $row = mysql_fetch_array(mysql_query("SELECT COUNT(*) FROM peers WHERE info_hash='".mysql_escape_string($info_hash)."' AND seeder"));
    $complete = $row[0];
    $incomplete = $total-$complete;
    return array("hash" => $info_hash,
		 "complete" => $complete,
		 "incomplete" => $incomplete);

  }

   function getTorrentList() {
     $list = array();
     $result = mysql_query("SELECT filename from torrents");
     while ($row = mysql_fetch_array($result)) {
       $list[] = $row[0];
     }
     return $list;
   }

   function getRawTorrent($torrent) {
     $result = mysql_query("SELECT raw_data FROM torrents WHERE filename='".mysql_escape_string($torrent)."'");
     if (mysql_num_rows($result)>0) {
       $row = mysql_fetch_row($result);
       return $row[0];
     } else
       return null;
   }
   function torrentExists($info_hash) {
     $result = mysql_query("SELECT COUNT(*) FROM torrents WHERE info_hash='".mysql_escape_string($info_hash)."'");
     $row = mysql_fetch_row($result);
     return $row[0]>0;
   }
  function getTorrentDetails($info_hash) {
    $peers = array();
    $now = time();
    $result = mysql_query("SELECT ip, port, UNIX_TIMESTAMP(time) AS time,  if (seeder,'seeder','leecher') AS what FROM peers WHERE info_hash = '".mysql_escape_string($info_hash)."'");
    while ($row = mysql_fetch_array($result)) {
      $ip = unpack("C*", $row['ip']);
      $ip = $ip[1] . '.' . $ip[2] . '.' . $ip[3] . '.*';
      $port = join('', unpack("n*", $row['port']));

      $peers[] = array("ip" => $ip,
		       "what" => $row['what'],
		       "port" => $port,
		       "time" => number_format( (int)(($now - $row['time'])/60)));
    }

    return $peers;
  }

  function addTorrentToTracker($tmpfile,$torrent) {
    $rawTorrent = file_get_contents($tmpfile);

    //Store the torrent on the filesystem, so if MySQL goes down, we
    //can keep on tracking
    parent::addTorrentToTracker($tmpfile,$torrent);
    $data = bdecode($rawTorrent);
    mysql_query("INSERT INTO torrents (info_hash,filename,raw_data) VALUES ('".mysql_escape_string($data['sha1'])."','".mysql_escape_string($torrent)."','".mysql_escape_string($rawTorrent)."')");
  }
   function deleteTorrent($torrent) {
     $result = mysql_query("SELECT info_hash FROM torrents WHERE filename='".mysql_escape_string($torrent)."'");
     if (mysql_num_rows($result)>0) {
       $row = mysql_fetch_row($result);
       $info_hash = $row[0];

       mysql_query("DELETE FROM peers WHERE info_hash='".mysql_escape_string($info_hash)."'");
       mysql_query("DELETE FROM torrents WHERE info_hash='".mysql_escape_string($info_hash)."'");
     }
     parent::deleteTorrent($torrent);
   }
  function setup() {
    global $settings;

    //We still need the flat file db set up to load the settings
    if (!parent::setup()) return false;
    if (isset($settings['mysql_database']) &&
	strlen($settings['mysql_database']) &&
	isset($settings['mysql_host']) &&
	strlen($settings['mysql_host']) &&
	isset($settings['mysql_username']) &&
	strlen($settings['mysql_username']) &&
	isset($settings['mysql_password']) &&
	@mysql_pconnect($settings['mysql_host'],$settings['mysql_username'],$settings['mysql_password'])) {
      //We can connect to the server. try to connect to the database
      if (!@mysql_selectdb($settings['mysql_database'])) {
	//If we can't connect to the database, try to create it
	@mysql_query("CREATE DATABASE IF NOT EXISTS ".$settings['mysql_database']);
	if (!@mysql_selectdb($settings['mysql_database']))
	  return false;
      }

      //We're connected to the database.

      //If we've used these settings before, we can be fairly certain
      //everything is OK
      if (isset($settings['mysql_verified']) && $settings['mysql_verified'])
	return true;

      //We haven't used these settings. Check to see if tables exist
      if (mysql_num_rows(mysql_query("SHOW TABLES")))
	return true;
      else { //We need to create the tables

	//This is the main torrent table
	mysql_query("CREATE TABLE torrents (
          info_hash char(40) NOT NULL,
          filename varchar(255) NOT NULL,
          raw_data MEDIUMBLOB NOT NULL,
          UNIQUE INDEX (filename),
          PRIMARY KEY(info_hash));");
	mysql_query("CREATE TABLE peers (
          info_hash char(40) NOT NULL,
          ip tinyblob NOT NULL,
          port tinyblob NOT NULL,
          seeder bool NOT NULL,
          time datetime NOT NULL,
          PRIMARY KEY (info_hash,ip(4),port(2)));");

	$this->addFlatFileTorrents();
	return mysql_num_rows(mysql_query("SHOW TABLES"));
      }
    } else
      return false;
  }

  function addFlatFileTorrents() {
     $oldTorrents = parent::getTorrentList();
     foreach ($oldTorrents as $torrent) {
       $hash = parent::getHashFromTorrent($torrent);
       $raw = parent::getRawTorrent($torrent);
       mysql_query("REPLACE INTO torrents (info_hash, filename,raw_data) VALUES ('".mysql_escape_string($hash)."','".mysql_escape_string($torrent)."','".mysql_escape_string($raw)."')");
     }
  }


  function setupHelpMessage() {
    //FIXME: make this nicer
    return 'MySQL is not setup properly';
  }

  function type() {
    return 'MySQL';
  }

}