<?php
require_once("legacylib.php");
require_once("datastore.php");
require_once("seeder.php");
session_cache_limiter('none');
session_start();

setup_data_directories();


function get_version() {
  return '0.92';
}

//Returns the base url of this tracker
function get_base_url() {
  
  if ((!isset($_SERVER['HTTPS'])) && $_SERVER['SERVER_PORT']!=80)
    $port = ':'.$_SERVER['SERVER_PORT'];
  elseif ((isset($_SERVER['HTTPS']) && $_SERVER['SERVER_PORT']!=443))
    $port = ':'.$_SERVER['SERVER_PORT'];
  else
    $port = '';
  return 'http'.(isset($_SERVER['HTTPS'])?'s':'').'://'.$_SERVER['HTTP_HOST'].$port.preg_replace('|^(.*/).*$|','\\1',$_SERVER['PHP_SELF']);
}

//Decodes a bEncoded string into a PHP structure
//See http://www.monduna.com/bt/faq.html#TERM_13
function bdecode($info) {
  $pos = 0;
  return bdecode_next($info,$pos);
}

//Decodes a bEncoded string, setting $pos to the the next token
function bdecode_next($info,& $pos) {
  //  print("Decoding ".substr($info,$pos,1)." at $pos <br/>\n");
    $origpos = $pos;
    switch (substr($info,$pos,1)) {
      case 'i':
	$pos = strpos($info,'e',$pos)+1;
	return substr($info,$origpos+1,$pos-$origpos-2);
      case 'd':
        $ret = array();
	$pos++;
        while (substr($info,$pos,1) != 'e') {
	  $key = bdecode_next($info,$pos);
	  $istart = $pos;
	  $val = bdecode_next($info,$pos);
	  $ilen = $pos - $istart;
	  $ret[$key] = $val;
	  if ($key == "info") {
	    $ret["sha1"] = sha1(substr($info, $istart, $ilen));
	  }

	}
	$pos++;
	return $ret;
      case 'l':
        $ret = array();
	$pos++;
        while (substr($info,$pos,1) != 'e') {
	  $ret[] = bdecode_next($info,$pos);
	}
	$pos++;
	return $ret;
      default:
	if (preg_match('/^(\d+):.*$/s',substr($info,$pos))) {
	  $len = preg_replace('/^(\d+):.*$/s','\\1',substr($info,$pos));
	  $pos += $len+1+strlen($len);
	  return substr($info,$origpos+strlen($len)+1,$pos-$origpos-strlen($len)-1);
	} else
	  return null; //There has been an error!
	
    }
}

//bEncodes a PHP structure
function bencode($struct) {
  if (is_array($struct)) {
    $out = 'd';
    ksort($struct);
    foreach($struct as $key => $value) {
      $out .= bencode($key);
      $out .= bencode($value);
    }
    $out .= 'e';
    return $out;
  } elseif (preg_match('/^(\+|\-)?\d+$/',$struct)) {
    return 'i'.$struct.'e';
  } else {
    return strlen($struct).':'.$struct;
  }

}

//Returns the name of the current user
function get_username() {
  if (isset($_SESSION['user']))
    return $_SESSION['user']['Username'];
  else
    return "";
}


//This should return the current user's password or a hash of their password
function get_passhash() {
  if (isset($_SESSION['user']))
    return $_SESSION['user']['Hash'];
  else
    return "";
}

//Returns hash of password
function hashpass($username,$password) {
  return(sha1($username.$password."downhillb==31337"));
}

//logs user in
//Returns true and sets $_SESSION['user'] on success, false on error
function login($username,$password) {
  global $store;
  $users=$store->getAllUsers();
  if ((isset($users[$username])) &&
      ($users[$username]['Hash']==hashpass($username,$password))) {
    $users[$username]['Username'] = $username;
    $_SESSION['user']=$users[$username];
    return true;
  } else
    return false;
}

function is_admin() {
  return (isset($_SESSION['user']) && $_SESSION['user'] && $_SESSION['user']['CanAdmin']);
}

function can_upload() {
  global $settings;

  return (($settings['NonAuthUpload']) ||
	  (isset($_SESSION['user']) && $_SESSION['user']['CanUpload']));
}

function site_title() {
  global $settings;
  return $settings['Title'];
}

function site_description() {
  global $settings;
  return $settings['Description'];
}
//logs user out
function logout() {
  $_SESSION['user'] = '';
  unset($_SESSION['user']);
  return true;
}

function setup_data_directories() {
  global $store;
  global $seeder;
  $store = new MYSqlStore();
  if (!$store->setup()) {
    $store = new FlatFileStore();
    if (!$store->setup()) {
      die($store->setupHelpMessage());
    }
  }
  $seeder = new ServerSideSeeder();
  $seeder->setup();
  return true;
}

function draw_first_time_link() {
  if (!file_exists('data/users')) {
    echo '<p>It looks like you\'re using Blog Torrent for the first time. You should <a href="newuser.php">create a new user account</a> before continuing.</p>';
  }
}

function draw_welcome() {?>
<script type="text/javascript"><!--
   if (hasWindows()) {
     document.writeln('<h2>How do I download a file?</h2>');
     document.writeln('<p>If you\'ve never used BitTorrent before, just click on the "Easy Downloader" link next to the file you want.  Open the Easy Downloader (by clicking "open" in your browser, or by saving it to your Desktop and double clicking on it).  That will install the Blog Torrent plugin and begin downloading the file you want. Note: if you already have a BitTorrent client installed, you do not need the Easy Downloader, just click on the .torrent file.</p>');
   } else if (hasMac()) {
     document.writeln('<h2>How do I download a file?</h2>');
     document.writeln('<p>If you\'ve never used BitTorrent before, just click on the "Easy Downloader" link next to the file you want.  Open the Easy Downloader (by clicking "open" in your browser, or by saving it to your Desktop and double clicking on it).  That will install the Blog Torrent plugin and begin downloading the file you want. Note: if you already have a BitTorrent client installed, you do not need the Easy Downloader, just click on the .torrent file.</p>');
   } else {
     document.writeln('<h2>How do I download a file?</h2>');
     document.writeln('<p>It looks like you\'re not running Windows or MacOS.  If that\'s true, then you can probably figure out how to install BitTorrent on your platform.  Go to <a href="http://bittorrent.com/">BitTorrent.com</a> for more information.</p>');
   }
--></script>
<noscript>
<p>If you've never used BitTorrent before and you're using Windows or Mac, just click on the "Easy Downloader" link next to the file you want.  Open the Easy Downloader (by clicking "open" in your browser, or by saving it to your Desktop and double clicking on it).  That will install the Blog Torrent plugin and begin downloading the file you want. Note: if you already have a BitTorrent client installed, you do not need the Easy Downloader, just click on the .torrent file.</p>
<p>If you're using another platform, you'll have to figure out how to install BitTorrent on your platform.  Go to <a href="http://bittorrent.com/">BitTorrent.com</a> for more information.</p>
</noscript>
<?php
  $username = get_username();
  if(strlen($username)>0) {
    echo "<p><b>Logged in as $username</b></p>\n";
  }
}

function draw_detect_scripts() {
?>
<script  language="JavaScript" type="text/javascript" ><!--
function hasBlogTorrent() {

<?php
  //Detects IE with the client installed
if (substr_count($_SERVER['HTTP_ACCEPT'],'application/x-blogtorrent')>0) {
  echo "return true;\n";
} else {
?>
  //Detects Mozilla browser with the client installed
  for (count=0;count<window.navigator.plugins.length;count++) {
    if (window.navigator.plugins[count].name.substring(0,12) == "Blog Torrent") {
      return true;
    }
  }

  //No client is installed
  return false;
<?php } ?>
}

function hasWindows() {
    return (navigator.userAgent.indexOf('Windows') > 0);
}
function hasMac() {
    return (navigator.userAgent.indexOf('Mac') > 0);
}
--></script>
<?php
}

function draw_upload_link() {
  $user = get_username();
  $hash = get_authhash($user,get_passhash());
?>
<script  language="JavaScript" type="text/javascript" ><!--
if (hasMac()) {
  document.writeln("<h1>Blog Torrent Upload OS X</h1>");

  if (hasBlogTorrent()) {

  document.writeln("<p>You have Blog Torrent installed (according to our detection). The upload window should have popped up automatically, but if you have pop-ups blocked click here:<br /><br /><strong><a href=\"trigger.php?hash=<?php echo $hash; ?>\">Start the Upload</a></strong></p>");

  document.writeln("<p>Once your upload begins, the file will be available to everyone who visits this site--as long as you keep your computer turned on and the Blog Torrent upload window open.</p><p>Want to upload another file?  You need to trigger a new upload: <a href=\"trigger.php\">Upload another file.</a> ");

  document.writeln("<p>If your Blog Torrent installation has deleted or damaged, download <a href=\"trigger.php?type=mac&hash=<?php echo $hash; ?>\">BlogTorrent.zip</a> to re-install.</p>");
  } else {
  document.writeln("<p>It looks like you haven't installed Blog Torrent yet. To do so, just download this small file: <a href=\"trigger.php?type=mac&hash=<?php echo $hash; ?>\">BlogTorrent.zip</a>. Double-click to start Blog Torrent-- it will ask you to pick the file you want to upload.</p>");

  document.writeln("<p>Once you do that, the file you chose will be available to everyone who visits this site--as long as you keep your computer turned on and the Blog Torrent window open.</p>");

  document.writeln("<p>If you're sure that you already have Blog Torrent, <a href=\"trigger.php?hash=<?php echo $hash; ?>\">this trigger</a> should start the upload</p>");
  }

  document.writeln("<br /><br /><p class=\"small\"><a href=\"trigger.php?type=exe&hash=<?php echo $hash; ?>\">The Windows installer</a> for your upload is available here if you need it.</p>");
} else {
  if (hasWindows()) {
    document.writeln("<h1>Blog Torrent Upload</h1>");
    
      if (hasBlogTorrent()) {

  document.writeln("<p>You have Blog Torrent installed (according to our detection). The upload window should have popped up automatically, but if you have pop-ups blocked click here:<br /><br /><strong><a href=\"trigger.php?hash=<?php echo $hash; ?>\">Start the Upload</a></strong></p>");

  document.writeln("<p>Once your upload begins, the file will be available to everyone who visits this site--as long as you keep your computer turned on and the Blog Torrent upload window open.</p><p>Want to upload another file?  You need to trigger a new upload: <a href=\"trigger.php\">Upload another file.</a> ");

  document.writeln("<p>If your Blog Torrent installation has deleted or damaged, download <a id=\"big_button\" href=\"trigger.php?type=exe&hash=<?php echo $hash; ?>\">Blog Torrent</a> and re-install.</p>");


   } else {

    document.writeln("<p>If this is your first time uploading at this site, you need to get Blog Torrent.</p>");

    document.writeln("<p><a id=\"big_button\" href=\"trigger.php?type=exe&hash=<?php echo $hash; ?>\">Click Here to Get Blog Torrent</a></p>");

    document.writeln("<p>Start the installation by clicking \"open\" in your browser (and saying \"Yes\" to all the warnings) or by saving the installer to your Desktop and double clicking on it.</p>");
    
    document.writeln("<p>Once you install Blog Torrent, it will ask you what file you want to upload.  Choose a file, and the upload will start.  Then you'll need to leave Blog Torrent running and your computer turned on so that other people can get the file.</p>");

    document.writeln("<p><b>Important: if you turn off your computer or close the Blog Torrent window, people (most likely) won't be able to get your file.</b></p>");

    document.writeln("<p>If you've already installed Blog Torrent (we can't detect your installation until you restart your web browser) then you don't have to download it again.  Just click here to start an upload:<Br /> <a href=\"trigger.php?hash=<?php echo $hash; ?>\">Start an Upload</a></p>");
    
    }
    
  document.writeln("<br /><br /><br /><p class=\"small\"><a href=\"trigger.php?type=mac&hash=<?php echo $hash; ?>\">The Mac installer</a> for your upload is available here if you need it.</p>");

      } else {
  document.writeln("<h1>Unsupported OS detected</h1>");
  document.writeln("<p>Since you're not using Windows or Mac, chances are you can figure out the Bit Torrent command line tool for creating torrents. (<a href=\"http://bittorrent.com/guide.html\">see this page</a>). Once you've created your torrent, you can upload it to this tracker using the form below. The announce URL for this server is <?php echo get_base_url().'announce.php'; ?>.</p>");

  document.writeln("<p>Blog Torrent makes this really easy, but there still isn't a version for your platform. You can help fix that.</p>");

  document.writeln("<p>The Blog Torrent team needs your donations to make versions for other platforms.  <a href=\"http://blogtorrent.com/donate/\">Donate and make it happen</a>.</p>");


  document.writeln("<form enctype=\"multipart/form-data\" action=\"upload.php\" method=\"post\">");
  document.writeln("<input type=\"hidden\" name=\"MAX_FILE_SIZE\" value=\"1000000\" />");
  document.writeln("<input name=\"Hash\" type=\"hidden\" value=\"<?php echo $hash; ?>\" />");
  document.writeln("<input name=\"BlogTorrentVersion\" type=\"hidden\" value=\"0.1\" />");
  document.writeln("<input name=\"Username\" type=\"hidden\" value=\"<?php echo $user; ?>\"/>Manual Upload: <input name=\"Torrent\" type=\"file\" /><input type=\"submit\" value=\"Go\"/>");
  document.writeln("</form>");
  document.writeln("<br /><br /><br /><p class=\"small\"><a href=\"trigger.php?type=exe&hash=<?php echo $hash; ?>\">The Windows installer</a> for your upload is available here if you need it, as is the <a href=\"trigger.php?hash=<?php echo $hash; ?>\">.blogtorrent file</a>, and <a href=\"trigger.php?type=mac&hash=<?php echo $hash; ?>\">The Mac installer</a>.</p>");
      
    }
}

function sendUpload() {
  if (hasBlogTorrent()) {
    window.open('trigger.php?hash=<?php echo $hash; ?>','download','width=300,height=200');
  }
}
--></script>
<noscript>
<h1>Uploading</h1>
<p>If this is your first time uploading at this site, you need to get Blog Torrent.</p>

<p><a id="big_button" href="trigger.php?type=exe&hash=<?php echo $hash; ?>">Click Here to Get Blog Torrent for Windows</a></p>
<p><a id="big_button" href="trigger.php?type=exe&hash=<?php echo $hash; ?>">Click Here to Get Blog Torrent for Mac</a></p>

<p>Start the installation by clicking "open" in your browser (and saying "Yes" to all the warnings) or by saving the installer to your Desktop and double clicking on it.</p>

<p>Once you install Blog Torrent, it will ask you what file you want to upload.  Choose a file, and the upload will start.  Then you'll need to leave Blog Torrent running and your computer turned on so that other people can get the file.</p>
<br /><br /><br />
<p>If you're certain you already installed Blog Torrent, you don't have to download it again.  Just <a href="trigger.php?hash=<?php echo $hash; ?>">click here</a> to start uploading (we can't detect Blog Torrent until you restart your web browser).</p>
<form enctype="multipart/form-data" action="upload.php" method="post">
<a href="trigger.php?hash=<?php echo $hash; ?>">Upload a File</a> |
<a href="trigger.php?type=exe&hash=<?php echo $hash; ?>">First Time</a> |
<input type="hidden" name="MAX_FILE_SIZE" value="1000000" />
<input name="Hash" type="hidden" value="<?php echo $hash; ?>" />
<input name="BlogTorrentVersion" type="hidden" value="0.1" />
<input name="Username" type="hidden" value="<?php echo $user; ?>"/>Manual: <input name="Torrent" type="file" /><input type="submit" value="Go"/>
</form>
</noscript>
<p><b>Hint:</b> If you upload to this site frequently drag the following direct link to the uploader into to your browser's toolbar: <a href="<?php echo get_base_url().'trigger.php';?>">Upload to <?php echo site_title();?></a></p>
<?php
}

function draw_download_link($filename) {
  $file = preg_replace('/^(.*)\.torrent$/','\\1',$filename);
?>
<script  language="JavaScript" type="text/javascript" ><!--
if (hasMac()) {
  document.writeln("<h1>How to download <?= htmlspecialchars($filename) ?> on Mac</h1>");

  if (hasBlogTorrent()) {
  document.writeln("<p>It looks like you already have Blog Torrent installed. Your download should have started. If you have pop-ups blocked, click <a href=\"btdownload.php?type=torrent&amp;file=<?= urlencode($filename) ?>\">the .torrent file</a> to start your download.</p>");
  document.writeln("<p>If your Blog Torrent install is damaged, you can reinstall it from <a href=\"btdownload.php?type=mac&amp;file=<?= urlencode($filename) ?>\">BlogTorrent.zip</a>.</p>");
  } else {
  document.writeln("<p>It looks like you haven't installed Blog Torrent, yet. To install it, you need to download a small file called <a href=\"btdownload.php?type=mac&amp;file=<?= urlencode($filename) ?>\">BlogTorrent.zip</a>. Save it to your desktop and double click on it to unstuff it. Then double click on the Blog Torrent application to begin downloading <?= htmlspecialchars($filename) ?> using Blog Torrent.</p>");

  document.writeln("<p>If you already have Blog Torrent or another BitTorrent client, click <a href=\"btdownload.php?type=torrent&amp;file=<?= urlencode($filename) ?>\">the .torrent file</a> to start your download.</p>");
  }

  document.writeln("<p>Blog Torrent is <a href=\"http://www.gnu.org/philosophy/free-sw.html\">Free Software</a>, made by volunteers, with no ads or spyware. If you'd like to learn more about Blog Torrent, check out their <a href=\"http://www.blogtorrent.com/\">homepage</a>.</p>");

  document.writeln("<p>Blog Torrent is a <a href=\"http://downhillbattle.org/\">Downhill Battle</a> project. We depend on your donations to continue developing software like this. <a href=\"http://blogtorrent.com/donate/\">Donate and help support this project</a>.</p>");
  document.writeln("<br /><br /><br /><p class=\"small\"><a href=\"btdownload.php?type=exe&amp;file=<?= urlencode($filename) ?>\">The Windows installer</a> for your download is available here if you need it.</p>");

} else {
  if (hasWindows()) {
    document.writeln("<h1>Downloading <?= htmlspecialchars($filename) ?></h1>");
    document.writeln("<p>If your download didn't start up, don't worry.  You just need to <a href=\"btdownload.php?type=exe&amp;file=<?= urlencode($filename) ?>\">get Blog Torrent</a>.</p>");
    document.writeln("<p>After the download, to start installation, click \"open\" and \"run\".  Since installing a program can be a security risk, you may be prompted with security warnings. Just click \"Yes\" to proceed.</p>");
    document.writeln("<p>Once you download and install Blog Torrent, it will automatically start fetching the file you want.</p>");


  document.writeln("<p>If you already have Blog Torrent or another BitTorrent client installed, <a href=\"btdownload.php?type=torrent&amp;file=<?= urlencode($filename) ?>\">the .torrent file</a> is available here.</p>");

    document.writeln("<p>Blog Torrent is <a href=\"http://www.gnu.org/philosophy/free-sw.html\">Free Software</a>, made by volunteers, with no ads or spyware.  If you'd like to learn more about Blog Torrent, check out their <a href=\"http://www.blogtorrent.com/\">homepage</a>.</p>");

    document.writeln("<p>Blog Torrent is a <a href=\"http://downhillbattle.org/\">Downhill Battle</a> project. We depend on your donations to continue developing software like this. <a href=\"http://blogtorrent.com/donate/\">Donate and help support this project</a>.</p>");

  document.writeln("<br /><br /><br /><p class=\"small\"><a href=\"btdownload.php?type=mac&amp;file=<?= urlencode($filename) ?>\">The Mac installer</a> for your download is available here if you need it.</p>");
} else {
    //Non-Windows, Non-Mac platform
    document.writeln("<h1>Downloading <?= htmlspecialchars($filename) ?></h1>");
    document.writeln("<p>Since you're not using Windows or Mac, chances are you can figure out Bit Torrent yourself, without the help of Blog Torrent. If you already have a BitTorrent client installed, <a href=\"btdownload.php?type=torrent&amp;file=<?= urlencode($filename) ?>\">this link</a> should automatically start your download.</p>");
    document.writeln("<p>Otherwise, you just need to <a href=\"http://bittorrent.com/download.html\">get a BitTorrent client for your platform</a>.</p>");

  document.writeln("<p>Blog Torrent makes the whole process much easier, but there isn't a version for your platform, yet. You can help fix that by <a href=\"http://blogtorrent.com/donate\">donating</a> or helping to port it yourself.</p>");

    document.writeln("<p>Blog Torrent is <a href=\"http://www.gnu.org/philosophy/free-sw.html\">Free Software</a>, made by volunteers, with no ads or spyware.  If you'd like to learn more about Blog Torrent, check out their <a href=\"http://blogtorrent.com\">homepage</a>.</p>");
    document.writeln("<p>Blog Torrent is a <a href=\"http://downhillbattle.org/\">Downhill Battle</a> project. We depend on your donations to continue developing software like this. <a href=\"http://blogtorrent.com/donate\">Donate and help support this project</a>.</p>");
  document.writeln("<br /><br /><br /><p class=\"small\"><a href=\"btdownload.php?type=exe&amp;file=<?= urlencode($filename) ?>\">The Windows installer</a> and <a href=\"btdownload.php?type=mac&amp;file=<?= urlencode($filename) ?>\">The Mac installer</a> for your download are available if you need either.</p>");

  }
}

function sendTorrent() {
  if (hasBlogTorrent()) {
    window.open('btdownload.php?type=torrent&file=<?= urlencode($filename) ?>','download','width=300,height=200');
  }
}
--></script>
<noscript>
<h1>Downloading <?= htmlspecialchars($filename) ?></h1>
<p>If your download didn't start up, don't worry.  You just need to get Blog Torrent</p>

<p><a href="btdownload.php?type=exe&amp;file=<?= urlencode($filename) ?>">Get Blog Torrent for Windows</a>.</p>

<p><a href="btdownload.php?type=mac&amp;file=<?= urlencode($filename) ?>">Get Blog Torrent for Mac</a>.</p>

<p>After the download, to start installation, click "open" and "run".  Since installing a program can be a security risk, you may be prompted with security warnings. Just click "Yes" to proceed.</p>

<p>Once you download and install Blog Torrent, it will automatically start fetching the file you want.</p>

<p>If you already have Blog Torrent or another BitTorrent client installed, <a href="btdownload.php?type=torrent&amp;file=<?= urlencode($filename) ?>">the .torrent file</a> is available here.</p>

<p>Blog Torrent is <a href="http://www.gnu.org/philosophy/free-sw.html">Free Software</a>, made by volunteers, with no ads or spyware.  If you'd like to learn more about Blog Torrent, check out their <a href="http://www.blogtorrent.com/">homepage</a>.</p>

<p>Blog Torrent is a <a href="http://downhillbattle.org/">Downhill Battle</a> project. We depend on your donations to continue developing software like this. <a href="http://blogtorrent.com/donate">Donate and help support this project</a>.</p>
</noscript>
<?php
}

//Sends an installer with file $tackon on the end
// if data is not null, sends data, only uses $tackon for the name
function send_installer($tackon,$data=null) {
  global $store;
  $magicNumber = 560097380;
  $file = "nsisinstaller.exe";

function packint32($int) {
  return pack("V",$int);
}
header("Content-Type: application/force-download");
header('Content-Disposition: attachment; filename="' . basename($tackon).".exe\"");

$buffer = file_get_contents($file);
 if (is_null($data))
   $torrent = $store->getRawTorrent($tackon);
 else
   $torrent = $data;
$len = strlen($torrent);
$crc = crc32($torrent);
$torrent .= packint32($crc).packint32($len).packint32($magicNumber);

readfile($file);
echo $torrent;

}

//Sends an installer with file $tackon on the end
// if data is not null, sends data, only uses $tackon for the name"
function send_mac_uploader($tackon,$data=null) {
  global $store;
  header("Content-type: application/octet-stream");
  header("Content-disposition: attachment; filename=BlogTorrent.zip");  

  if (file_exists('macclient.obj'))
     $zip = unserialize( file_get_contents('macclient.obj'));
   else
     $zip = unserialize( file_get_contents('data/macclient.obj'));

 if (is_null($data))
   $torrent = $store->getRawTorrent($tackon);
 else
   $torrent = $data;

  $zip->add_file($torrent,'Blog Torrent.app/Contents/EasyUpload.blogtorrent');

  echo $zip->file();

}


//Sends an installer with file $tackon on the end
// if data is not null, sends data, only uses $tackon for the name"
function send_mac_installer($tackon,$data=null) {
  global $store;
  header("Content-type: application/octet-stream");
  header("Content-disposition: attachment; filename=BlogTorrent.zip");  

  if (file_exists('macclient.obj'))
     $zip = unserialize( file_get_contents('macclient.obj'));
   else
     $zip = unserialize( file_get_contents('data/macclient.obj'));

 if (is_null($data))
   $torrent = $store->getRawTorrent($tackon);
 else
   $torrent = $data;

  $zip->add_file($torrent,'Blog Torrent.app/Contents/EasyDownload.torrent');

  echo $zip->file();

}

//FIXME all of the functions below this point are depricated
//      Use $store instead

function getstat($file)
{
  global $store;
  $store->getStat($file);
}

//The following functions generate and verify authentication hashes
//given to the client.  Using authentication hashes lets us keep all
//of the authentication on the server side. The client only has to
//keep the hash, then send it back. It never has to deal with
//passwords. The "hash" file is a bEncoded associative array.  The
//indexes to the array are authentication hashes and usernames hashed
//together. The values are timestamps.

//We hash the authentication hash with the username, so that if
//someone downloads the hash file somehow, they can't just copy the
//hashes and upload files without authentication.

function is_valid_authhash($username,$hash) {
  global $store;
  return $store->isValidAuthHash($username,$hash);
}

function get_authhash($username, $passhash) {
  global $store;
  return $store->getAuthHash($username,$passhash);
}

function drop_authhash($username,$hash) {
  global $store;
  return $store->dropAuthHash($username,$hash);
}

//Private function used by add_hash and drop_hash
function clear_old_authhashes(& $hashes) {
  global $store;
  $store->clearOldAuthHashes($hashes);
}

//Sends the location of this feed to BlogTorrent.com periodically
function phone_home() {
  global $store;
  return $store->phoneHome();
}

function add_torrent_to_tracker($filename) {
  global $store;
  return $store->addTorrentToTracker($filename);
}


function get_all_users () {
  global $store;
  return $store->getAllUsers();
}

function delete_torrent($torrent) {
  global $store;
  $store->deleteTorrent($torrent);
}

function delete_user($username) {
  global $store;
  return $store->deleteUser($username);
}

//New users are stored as a list of hashes
function add_new_user($username,$password,$email, &$error) {
  global $store;
  return $store->addNewUser($username,$password,$email, $error);
}

function auth_new_user($hashlink,$username) {
  global $store;
  return $store->authNewUser($hashlink,$username);
}

function change_password($password) {
  global $store;
  return $store->changePassword($password);
}

//Adds/Updates a user in the user database
function update_user($username,$hash,$email,$canUpload=false,$canAdmin=false) {
  global $store;
  return $store->updateUser($username,$hash,$email,$canUpload,$canAdmin);
}

//Loads settings from config file
//Returns true on success, false on failure
function load_settings() {
  global $store;
  return $store->loadSettings();
}

//Saves settings to config file
//Returns true on success, false on failure
function save_settings($newsettings) {
  global $store;
  return $store->saveSettings($newsettings);
}

